package com.gxzc.zen.orm.config

import com.alibaba.druid.pool.xa.DruidXADataSource
import com.gxzc.zen.orm.DynamicMultipleDataSource
import com.gxzc.zen.orm.contants.DSKey
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Qualifier
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.boot.context.properties.ConfigurationProperties
import org.springframework.boot.jta.atomikos.AtomikosDataSourceBean
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.DependsOn
import org.springframework.context.annotation.Primary
import javax.sql.DataSource
import javax.sql.XADataSource


/**
 * 多数据源配置
 * @author NorthLan at 2018/1/29
 */
@Configuration
class MultipleDataSourceConfig {
    companion object {
        private val logger = LoggerFactory.getLogger(MultipleDataSourceConfig::class.java)
    }

    init {
        logger.info("MultipleDataSourceConfig initializing...")
    }

    @Bean(DSKey.DSKEY_SYS + "druid")
    @ConfigurationProperties(prefix = "datasource.sys")
    fun dataSourceSysDruid(): XADataSource {
        return DruidXADataSource()
    }

//    @Bean(DSKey.DSKEY_SHARED + "druid")
//    @ConfigurationProperties(prefix = "datasource.shared")
//    @ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "true", matchIfMissing = true)
//    fun dataSourceSharedDruid(): XADataSource {
//        return DruidXADataSource()
//    }

    @Bean(DSKey.DSKEY_SYS)
    @DependsOn(DSKey.DSKEY_SYS + "druid")
    @Primary
    fun dataSourceSys(): DataSource {
        return AtomikosDataSourceBean().also {
            it.xaDataSource = dataSourceSysDruid()
        }
    }

//    @Bean(DSKey.DSKEY_SHARED)
//    @DependsOn(DSKey.DSKEY_SHARED + "druid")
//    @ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "true", matchIfMissing = true)
//    fun dataSourceShared(): DataSource {
//        return AtomikosDataSourceBean().also {
//            it.xaDataSource = dataSourceSharedDruid()
//        }
//    }

    @Bean
    @ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "true", matchIfMissing = true)
    fun multipleDataSource(@Qualifier(DSKey.DSKEY_SYS) dataSourceSys: DataSource): DynamicMultipleDataSource {
        return DynamicMultipleDataSource().apply {
            setTargetDataSources(mapOf(
                    DSKey.DSKEY_SYS to dataSourceSys))
            setDefaultTargetDataSource(dataSourceSys)
        }
    }
}